@@ -59,9 +59,11 @@ public class Hand
    boolean forced_ewmp;
    int ewavetype;
    int nsavetype;
+	Board board;
    public Hand()
    {
    }
+	@SuppressWarnings("fallthrough")
    public Hand(String[] data) throws HandParseException
    {
       if (data.length < 4) {
@@ -100,7 +102,8 @@ public class Hand
                } catch (NumberFormatException NFe) { 
                   if (Debug.debug) Debug.print(NFe);
                   String av = data[7].toLowerCase();
-                  if (av.startsWith("av") && av.length() == 3)
+                  if (av.startsWith("av") && av.length() == 3) {
+							if (Debug.debug) Debug.print("Parsing average, mode character is "+av.charAt(2));
                      switch (av.charAt(2)) {
                         case '=':
                            ewavetype = AVERAGE;
@@ -114,6 +117,7 @@ public class Hand
                         default:
                            throw new HandParseException(_("Invalid Average: ")+data[7]);
                      }
+						}
                   else throw new HandParseException(_("Invalid Score: ")+data[7]);
                }
             }
@@ -124,7 +128,8 @@ public class Hand
                } catch (NumberFormatException NFe) {
                   if (Debug.debug) Debug.print(NFe);
                   String av = data[6].toLowerCase();
-                  if (av.startsWith("av") && av.length() == 3)
+                  if (av.startsWith("av") && av.length() == 3) {
+							if (Debug.debug) Debug.print("Parsing average, mode character is "+av.charAt(2));
                      switch (av.charAt(2)) {
                         case '=':
                            nsavetype = AVERAGE;
@@ -138,6 +143,7 @@ public class Hand
                         default:
                            throw new HandParseException(_("Invalid Average: ")+data[6]);
                      }
+						}
                   else throw new HandParseException(_("Invalid Score: ")+data[6]);
                }
             }
@@ -153,6 +159,7 @@ public class Hand
                this.declarer = data[4].charAt(0);
       }
    }
+	@SuppressWarnings("fallthrough")
    public void score() throws ScoreException, ContractParseException, HandParseException
    {
       String[] n = number.split(":");
@@ -241,6 +248,7 @@ public class Hand
       }
    }
 
+	@SuppressWarnings("fallthrough")
    public void check() throws HandParseException
    {
       try {
@@ -282,12 +290,14 @@ public class Hand
    public char getDeclarer() { return declarer; }
    public boolean isAveraged() { return !(0 == ewavetype && 0 == nsavetype); }
    public int getEWAverage() { return ewavetype; }
-   public int getNSAverage() { return ewavetype; }
+   public int getNSAverage() { return nsavetype; }
    public boolean hasForcedNSMP() { return forced_nsmp; }
    public boolean hasForcedEWMP() { return forced_ewmp; }
    public boolean hasForcedNSScore() { return forced_nsscore; }
    public boolean hasForcedEWScore() { return forced_ewscore; }
+	public Board getBoard() { return board; }
 
+	public void setBoard(Board b) { board = b; }
    public void setNSMP(double mp) { nsmp = mp; }
    public void setEWMP(double mp) { ewmp = mp; }
    public void setNumber(String number) { this.number = number; }
