@@ -23,6 +23,7 @@ import cx.ath.matthew.debug.Debug;
 
 import java.util.Arrays;
 import java.util.List;
+import java.util.Vector;
 import java.text.DecimalFormat;
 import java.text.FieldPosition;
 
@@ -34,6 +35,7 @@ public class Pair
    double mps;
    double percentage;
    double lps;
+	Vector<Hand> hands = new Vector<Hand>();
    public static synchronized int getMaxNames() { return maxnames; }
    public static synchronized void resetNames() { maxnames = 0; }
    public Pair() {}
@@ -69,19 +71,19 @@ public class Pair
          if (maxnames < names) maxnames = names;
       }
    }
-   public void total(List boards)
+   public void total(List<Board> boards)
    {
       double top = 0;
       int bds = 0;
       mps = 0;
-      for (Board b: (Board[]) boards.toArray(new Board[0])) {
+      for (Board b: boards) {
          top = b.getTop();
          if (b.played(number)) {
             mps += b.getMPs(number);
             bds++;
          }
       }
-      percentage = (mps*100) / (top*bds);
+      percentage = (mps*100.0) / (top*bds);
       if (Debug.debug) Debug.print("totalling for pair "+number+": mps="+mps+", boards="+bds+", top="+top+", %age="+percentage);
    }
    public String getNumber() { return number; }
@@ -139,5 +141,13 @@ public class Pair
 
       return rv;
    }
+	public void addHand(Hand h) 
+	{
+		hands.add(h);
+	}
+	public List<Hand> getHands()
+	{
+		return hands;
+	}
 }
 
